// $Id: encode.c,v 1.2 2006/02/09 19:14:25 Daniel.May Exp $
//
// FIX Adapted for STreaming (sm) (FAST Protocol (sm)) 
//
// Copyright (c) 2005-2006, Pantor Engineering AB (http://www.pantor.com)
// Copyright (c) 2005-2006, SpryWare LLC (http://www.spryware.com)
// Copyright (c) 2005-2006, FIX Protocol Ltd (http://www.fixprotocol.org)
// All Rights Reserved.
//
// This work is distributed under the W3C Software License [1] in the
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY, SATISFACTORY QUALITY or FITNESS 
// FOR A PARTICULAR PURPOSE.
//
// [1] http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231
// [FPL's Intellectual Property details] http://www.fixprotocol.org/ip
// [FAST Protocol details] http://www.fixprotocol.org/fast
// [FAST Protocol credits] http://fixprotocol.org/fastcredits
//
// The example application 'encode' will encode a sample data feed into the FAST format.
// The data feed is a very simple feed containing only 6 fields, SeqNum, Time, Symbol,Price, Size, Exch
// The data is read from stdin a line at a time, parsed, then encoded into fast messages.  The sample input
// file "example.dat" contains a small sample data set.  Ouptut is written to stdout.  See the sample test
// scripts example.sh (Linux) or example.bat (Windows) for example usage.
//

/**
 * @example encode.c
 * FIX Adapted for STreaming API decoder example
 */

#include "fastapi.h"
#include "common.h"

// define the template ID
enum test_tids
{
   TEST_BASE_TID = 0,
};

// define the fields
enum test_fields
{
   TID          = MAKE_TAG (FAST_TYPE_U32,  FAST_OP_COPY,  TEST_BASE_TID,  0),
   SEQ_NUM      = MAKE_TAG (FAST_TYPE_U32,  FAST_OP_INCR,  TEST_BASE_TID,  1),
   TIME         = MAKE_TAG (FAST_TYPE_U32,  FAST_OP_COPY,  TEST_BASE_TID,  2),
   SYMBOL       = MAKE_TAG (FAST_TYPE_STR,  FAST_OP_COPY,  TEST_BASE_TID,  3),
   PRICE        = MAKE_TAG (FAST_TYPE_U32,  FAST_OP_COPY,  TEST_BASE_TID,  4),
   SHARES       = MAKE_TAG (FAST_TYPE_U32,  FAST_OP_COPY,  TEST_BASE_TID,  5),
   EXCH         = MAKE_TAG (FAST_TYPE_STR,  FAST_OP_COPY,  TEST_BASE_TID,  6),
};


/**
 * The example application 'encode' will encode a sample data feed into the FAST format.
 * The data feed is a very simple feed containing only 6 fields, SeqNum, Time, Symbol,Price, Size, Exch
 */
int main(int argc, char* argv[])
{
   char data [128];
   fast_codec_t *codec;

	// We want binary input from stdio and stdout 
    init_platform_io();

	// Create the Codec
	codec = fast_create_codec ();
	assert(NULL!=codec);

    // Inside some IDE's, it may be ease debugging to set the input file here ...
	//freopen( "../example.dat", "r", stdin );

	// Encode the data
	while(fgets(data,sizeof(data)-1,stdin) != NULL)
	{
		// ; SeqNum, Time, Symbol,Price, Size, Exch
		// skip comment header
		if(';' == data[0])
			continue;
		
		// encode the data, we always start off with a template ID
        fast_encode_new_msg (codec, TID);
		fast_encode_u32 (codec, TID, 1);
		// SeqNum is first 6 chars in input file, starting at pos 0
		fast_encode_u32 (codec, SEQ_NUM, fast_ascii_to_u32((u8 *)&data[0],6));				
		// Time is next 6 chars in input file, starting at pos 7
		fast_encode_u32 (codec, TIME, fast_ascii_to_u32((u8 *)&data[7],6));				
		// Symbol is next 3 chars in input file, starting at pos 14
		fast_encode_str (codec, SYMBOL, &data[14],3);				
		// Price is next 5 chars in input file, starting at pos 18
		fast_encode_u32 (codec, PRICE, fast_ascii_to_u32((u8 *)&data[18],5));				
		// Shares are next 3 chars in input file, starting at pos 24
		fast_encode_u32 (codec, SHARES, fast_ascii_to_u32((u8 *)&data[24],3));				
		// Exchange is next char in input file, starting at pos 28
		fast_encode_str (codec, EXCH, &data[28],1);

		// we are done with this message
		if(fast_encode_end_msg (codec,0) != FAST_ERR_NONE)
			fast_print_error (codec, stderr);
   }

	// clean up
	fast_destroy_codec (codec);
	return 0;
}


